<?php
/**
* @package RSForm!Pro
* @copyright (C) 2007-2019 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

// no direct access
defined('_JEXEC') or die('Restricted access');

class plgSystemRsfprsmail extends JPlugin
{
	protected $autoloadLanguage = true;

	public function onRsformFormSave($form)
	{
		$app                    = JFactory::getApplication();
		$data                   = $app->input->post->get('rsm', array(), 'array');
		$data['rsm_merge_vars'] = $app->input->post->get('rsm_merge_vars', array(), 'array');
		$data['form_id']        = $form->FormId;

		$row = $this->getTable();
		if ($row)
		{
			if (!$row->save($data))
			{
				JFactory::getApplication()->enqueueMessage($row->getError(), 'error');
			}
		}
	}

	protected function getTable()
	{
		return JTable::getInstance('RSForm_Rsmail', 'Table');
	}
	
	public function onRsformBackendFormCopy($args)
	{
		$formId = $args['formId'];
		$newFormId = $args['newFormId'];

		if ($row = $this->getTable() )
		{
			if ($row->load($formId))
			{
				if (!$row->bind(array('form_id'=>$newFormId)))
				{
					return false;
				}

				$row->check();

				return $row->store();
			}
		}
	}
	
	public function onRsformBackendAfterShowFormEditTabs()
	{
		if (!file_exists(JPATH_SITE.'/components/com_rsmail/helpers/actions.php'))
		{
			return false;
		}

		require_once JPATH_SITE.'/components/com_rsmail/helpers/actions.php';
		$formId = JFactory::getApplication()->input->getInt('formId');
		$row    = $this->getTable();

		if (!$row)
		{
			return false;
		}

		$row->load($formId);
		$row->rsm_merge_vars = @unserialize($row->rsm_merge_vars);
		if ($row->rsm_merge_vars === false)
		{
			$row->rsm_merge_vars = array();
		}
		
		// Get a new instance of the RSMail! helper
		$rsmail = new rsmHelper(); 
		
		// Fields
		$fields_array = $this->_getFields($formId);
		$fields = array();
		$fields[] = JHtml::_('select.option', '', JText::_('RSFP_RSM_IGNORE'));
		foreach ($fields_array as $field)
		{
			$fields[] = JHtml::_('select.option', $field, $field);
		}

		// Action
		$rsm_action = array(
			JHtml::_('select.option', 1, JText::_('RSFP_RSM_ACTION_SUBSCRIBE')),
			JHtml::_('select.option', 0, JText::_('RSFP_RSM_ACTION_UNSUBSCRIBE')),
			JHtml::_('select.option', 2, JText::_('RSFP_RSM_LET_USER_DECIDE'))
		);
		$lists['rsm_action'] = JHtml::_('select.genericlist', $rsm_action, 'rsm[rsm_action]', 'onchange="rsmailChangeAction(this.value);"', 'value', 'text', $row->rsm_action);

		// Action Field
		$lists['rsm_action_field'] = JHtml::_('select.genericlist', $fields, 'rsm[rsm_action_field]', $row->rsm_action != 2 ? 'disabled="disabled"' : '', 'value', 'text', $row->rsm_action_field);

		// RSMail! Lists
		$results = $rsmail->getLists();
		$rsm_lists = array(
			JHtml::_('select.option', '0', JText::_('RSFP_PLEASE_SELECT_LIST'))
		);
		if (!empty($results))
		{
			foreach ($results as $result)
			{
				$rsm_lists[] = JHtml::_('select.option', $result->IdList, $result->ListName);
			}
		}

		$lists['rsm_list_id'] = JHtml::_('select.genericlist', $rsm_lists, 'rsm[rsm_list_id]', 'onchange="rsmailChangeList(this.value)"', 'value', 'text', $row->rsm_list_id);

		// Merge Vars
		$merge_vars = array();
		if ($row->rsm_list_id)
		{
			$rsm_fields = $rsmail->getFields($row->rsm_list_id);
			$merge_vars['rsm_email'] = JText::_('RSFP_RSM_EMAIL');
			if (!empty($rsm_fields))
			{
				foreach ($rsm_fields as $field)
				{
					$merge_vars[$field] = $field;
				}
			}
		}

		$lists['fields'] = array();
		if (is_array($merge_vars))
		{
			foreach ($merge_vars as $merge_var => $title)
			{
				$lists['fields'][$merge_var] = JHtml::_('select.genericlist', $fields, 'rsm_merge_vars['.$merge_var.']', null, 'value', 'text', isset($row->rsm_merge_vars[$merge_var]) ? $row->rsm_merge_vars[$merge_var] : null);
			}
		}

		$rsm_action_type = array(
			JHtml::_('select.option', 1, JText::_('RSFP_RSM_ACTION_TYPE_ERROR')),
			JHtml::_('select.option', 2, JText::_('RSFP_RSM_ACTION_TYPE_UPDATE')),
			JHtml::_('select.option', 3, JText::_('RSFP_RSM_ACTION_TYPE_IGNORE'))
		);

		$lists['published']     = RSFormProHelper::renderHTML('select.booleanlist', 'rsm[rsm_published]', '', $row->rsm_published);
		$lists['confirm'] 	    = RSFormProHelper::renderHTML('select.booleanlist', 'rsm[rsm_confirm]', '', $row->rsm_confirm);
		$lists['action_type']	= JHtml::_('select.genericlist', $rsm_action_type, 'rsm[rsm_action_type]', '', 'value', 'text', $row->rsm_action_type);
		?>
		<div id="rsmaildiv">
			<script type="text/javascript">
				function rsmailChangeList(value) {
					stateLoading();

					var params = [];
					params.push('list_id=' + encodeURIComponent(value));

					var xml = buildXmlHttp();
					var url = 'index.php?option=com_rsform&task=plugin&plugin_task=rsmail.get_merge_vars';
					xml.open("POST", url, true);

					params = params.join('&');

					//Send the proper header information along with the request
					xml.setRequestHeader("Content-type", "application/x-www-form-urlencoded");

					xml.send(params);
					xml.onreadystatechange=function() {
						if (xml.readyState === 4) {
							var table = document.getElementById('tableRsmailMergeVars');
							while (table.rows.length >= 1) {
                                table.deleteRow(table.rows.length - 1);
							}

							if (xml.responseText !== '') {
								var result = xml.responseText.split("\n");
								var field, property;
								var x, y, z, i;
								for (i = 0; i < result.length; i++) {
									field = result[i];
									property = result[i];

									if (field.indexOf('rsm_email_') > -1) {
										field = field.replace('rsm_email_','');
										property = 'rsm_email';
									}

									x = table.insertRow(table.rows.length);
									y = x.insertCell(0);
									y.innerHTML = field;
									y.nowrap = true;
									z = x.insertCell(1);

									var select = document.createElement('select');
									select.name = 'rsm_merge_vars[' + property + ']';
									<?php foreach ($fields as $field) { ?>
									var option = document.createElement('option');
									option.text = <?php echo json_encode($field->text); ?>;
									option.value = <?php echo json_encode($field->value); ?>;
									select.options.add(option);
									<?php } ?>
									z.appendChild(select);
								}
							}

							stateDone();
						}
					}
				}

				function rsmailChangeAction(value) {
					if (value === '1' || value === '2') {
						document.getElementById('subscribeOption').style.display = '';
					} else {
						document.getElementById('subscribeOption').style.display = 'none';
					}

					document.getElementsByName('rsm[rsm_action_field]')[0].disabled = value !== '2';
				}
			</script>

			<p><?php echo JHtml::_('image', 'administrator/components/com_rsform/assets/images/rsmail_logo.png', 'RSMail!'); ?></p>
			<div class="alert alert-info"><?php echo JText::_('RSFP_RSMAIL_DESC'); ?></div>
			<table class="table table-bordered">
				<tr>
					<td><?php echo JText::_('RSFP_RSM_USE_INTEGRATION'); ?></td>
					<td><?php echo $lists['published']; ?></td>
				</tr>
				<tr>
					<td>
						<span class="hasTip" title="<?php echo JText::_('RSFP_RSM_ACTION_DESC'); ?>"><?php echo JText::_('RSFP_RSM_ACTION'); ?></span>
					</td>
					<td nowrap="nowrap"><?php echo $lists['rsm_action']; ?> <?php echo $lists['rsm_action_field']; ?></td>
				</tr>
				<tr id="subscribeOption">
					<td><?php echo JText::_('RSFP_RSM_SUBHSCRIBER_EXISTS'); ?></td>
					<td><?php echo $lists['action_type']; ?>
						<p><?php echo JText::_('RSFP_RSM_ACTION_WARNING'); ?></p></td>
				</tr>
				<tr>
					<td><?php echo JText::_('RSFP_RSM_LIST_ID'); ?></td>
					<td nowrap="nowrap"><?php echo $lists['rsm_list_id']; ?></td>
				</tr>
				<tr>
					<td><?php echo JText::_('RSFP_RSM_ONLY_AFTER_CONFIRM'); ?></td>
					<td><?php echo $lists['confirm']; ?>
						<p><?php echo JText::_('RSFP_RSM_ONLY_AFTER_CONFIRM_WARNING'); ?></p></td>
				</tr>
			</table>

			<h3 class="rsfp-legend"><?php echo JText::_('RSFP_RSM_MERGE_VARS'); ?></h3>
			<p><?php echo JText::_('RSFP_RSM_MERGE_VARS_DESC'); ?></p>
			<table class="table table-bordered">
				<tbody id="tableRsmailMergeVars">
				<?php if (is_array($merge_vars)) { ?>
					<?php foreach ($merge_vars as $merge_var => $title) { ?>
						<tr>
							<td nowrap="nowrap"><?php echo $title; ?></td>
							<td><?php echo $lists['fields'][$merge_var]; ?></td>
						</tr>
					<?php } ?>
				<?php } ?>
				</tbody>
			</table>
			<script type="text/javascript">rsmailChangeAction();</script>
		</div>
		<?php

		JFactory::getDocument()->addScriptDeclaration('jQuery(function(){ rsmailChangeAction(\'' . $row->rsm_action . '\'); });');
	}
	
	public function onRsformBackendAfterShowFormEditTabsTab()
	{
		echo '<li><a href="javascript: void(0);" id="rsmail"><span class="rsficon rsficon-envelope-o"></span><span class="inner-text">'.JText::_('RSFP_RSMAIL_INTEGRATION').'</span></a></li>';
	}
	
	protected function _getSubmission($SubmissionId)
	{
		static $cache = array();
		
		if (!isset($cache[$SubmissionId]))
		{
			$db 	= JFactory::getDbo();
			$query 	= $db->getQuery(true);
			
			$query->select('*')
				  ->from($db->qn('#__rsform_submissions'))
				  ->where($db->qn('SubmissionId').'='.$db->q($SubmissionId));
			$cache[$SubmissionId] = $db->setQuery($query)->loadObject();
		}
		
		return $cache[$SubmissionId];
	}
	
	protected function _getRow($formId)
	{
		static $cache = array();
		
		if (!isset($cache[$formId]))
		{
			$db 	= JFactory::getDbo();
			$query 	= $db->getQuery(true);
			
			$query->select('*')
				  ->from($db->qn('#__rsform_rsmail'))
				  ->where($db->qn('form_id').'='.$db->q($formId))
				  ->where($db->qn('rsm_published').'='.$db->q(1));
			$cache[$formId] = $db->setQuery($query)->loadObject();
			
			$row =& $cache[$formId];
			if ($row)
			{
				$row->rsm_merge_vars = @unserialize($row->rsm_merge_vars);
				if ($row->rsm_merge_vars === false)
				{
					$row->rsm_merge_vars = array();
				}
			}
		}
		
		return $cache[$formId];
	}
	
	protected function _transformData($SubmissionId)
	{
		$db 		= JFactory::getDbo();
		$query		= $db->getQuery(true);
		$submission = $this->_getSubmission($SubmissionId);
		$formId		= $submission->FormId;
		
		$query->select($db->qn('FieldName'))
			  ->select($db->qn('FieldValue'))
			  ->from($db->qn('#__rsform_submission_values'))
			  ->where($db->qn('SubmissionId').'='.$db->q($SubmissionId));
		$submission->values = $db->setQuery($query)->loadObjectList('FieldName');
		
		$db->setQuery("SELECT p.PropertyValue AS name, c.ComponentId, c.ComponentTypeId, ct.ComponentTypeName, c.Order FROM #__rsform_properties p LEFT JOIN #__rsform_components c ON (c.ComponentId=p.ComponentId) LEFT JOIN #__rsform_component_types ct ON (ct.ComponentTypeId=c.ComponentTypeId) WHERE c.FormId='".$formId."' AND p.PropertyName='NAME' AND c.Published='1' ORDER BY c.Order");
		$components = $db->loadObjectList();

		$data = array();
		
		foreach ($components as $component)
		{
			if (isset($submission->values[$component->name]))
			{
				$data[$component->name] = $submission->values[$component->name]->FieldValue;
				if (in_array($component->ComponentTypeId, array(RSFORM_FIELD_SELECTLIST, RSFORM_FIELD_CHECKBOXGROUP)))
				{
					$data[$component->name] = explode("\n", $data[$component->name]);
				}
			}
			else
			{
				$data[$component->name] = '';
			}
		}
		
		return $data;
	}
	
	protected function _getAction($row, $form)
	{
		$subscribe = null;

		if ($row->rsm_action == 1)
		{
			$subscribe = true;
		}
		elseif ($row->rsm_action == 0)
		{
			$subscribe = false;
		}
		elseif ($row->rsm_action == 2 && isset($form[$row->rsm_action_field]))
		{
			if (is_array($form[$row->rsm_action_field]))
			{
				foreach ($form[$row->rsm_action_field] as $i => $value)
				{
					$value = strtolower(trim($value));
					if ($value == 'subscribe')
					{
						$subscribe = true;
						break;
					}
					elseif ($value == 'unsubscribe')
					{
						$subscribe = false;
						break;
					}
				}
			}
			else
			{
				$form[$row->rsm_action_field] = strtolower(trim($form[$row->rsm_action_field]));
				if ($form[$row->rsm_action_field] == 'subscribe')
				{
					$subscribe = true;
				}
				elseif ($form[$row->rsm_action_field] == 'unsubscribe')
				{
					$subscribe = false;
				}
			}
		}
		
		return $subscribe;
	}
	
	protected function _doAction($row, $SubmissionId, $form = null)
	{
		if (file_exists(JPATH_SITE.'/components/com_rsmail/helpers/actions.php'))
		{
			require_once JPATH_SITE.'/components/com_rsmail/helpers/actions.php';
			
			// Get a new instance of the RSMail! helper
			$rsmail = new rsmHelper(); 
			
			if (isset($row->rsm_merge_vars['rsm_email']))
			{
				if (!$form)
				{
					$form = $this->_transformData($SubmissionId);
				}

				$email      = isset($form[$row->rsm_merge_vars['rsm_email']]) ? $form[$row->rsm_merge_vars['rsm_email']] : '';
				$merge_vars = array();

				foreach ($row->rsm_merge_vars as $tag => $field)
				{
					if (empty($tag) || $tag == 'rsm_email')
					{
						continue;
					}
					
					if (!isset($form[$field]))
					{
						$form[$field] = '';
					}
					
					if (is_array($form[$field]))
					{
						array_walk($form[$field], array($this, '_escapeCommas'));
						$form[$field] = implode(',', $form[$field]);
					}
					
					$merge_vars[$tag] = $form[$field];
				}
				
				// Subscribe action - Subscribe, Unsubscribe or Let the user choose
				$subscribe = $this->_getAction($row, $form);
				
				// Prepare list
				$list = $rsmail->setList($row->rsm_list_id, $merge_vars);
				
				/* $subscribe = true - subscribe user
				$subscribe = false - unsubscribe user
				$subscribe = null - do nothing */
				if ($subscribe === true)
				{
					if ($rsmail->isSubscribed($email, $row->rsm_list_id) && $row->rsm_action_type == 3)
					{
						return;
					}
					
					$state = $rsmail->getState();
					
					if ($row->rsm_action_type == 2)
					{
						$db = JFactory::getDbo();
						$query = $db->getQuery(true)
							->select($db->qn('published'))
							->from($db->qn('#__rsmail_subscribers'))
							->where($db->qn('SubscriberEmail') . ' = ' . $db->q($email))
							->where($db->qn('IdList') . ' = ' . $db->q($row->rsm_list_id));
						$db->setQuery($query);
						$user_state = $db->loadResult();
						
						if (!is_null($user_state))
						{
							$state = $user_state;
						}
						
						$confirmation = rsmailHelper::getMessage('confirmation');
						if (!$confirmation->enable)
						{
							$state = 1;
						}
					}

					require_once JPATH_ADMINISTRATOR . '/components/com_rsform/helpers/submissions.php';
					$submission = RSFormProSubmissionsHelper::getSubmission($SubmissionId, false);
					
					// Subscribe user
					$idsubscriber = $rsmail->subscribe($email, $list, $state, $submission->UserId, true);
					
					if ($idsubscriber)
					{
						// The user must confirm his subscription
						if (!$state)
						{
							$hash = md5($row->rsm_list_id.$idsubscriber.$email);
							$rsmail->confirmation($row->rsm_list_id, $email, $hash);
						}
						
						// Send notifications
						if ($row->rsm_action_type != 2)
						{
							$rsmail->notifications($row->rsm_list_id, $email, $merge_vars);
						}
					}
				}
				elseif ($subscribe === false)
				{
					$rsmail->unsubscribeLink($email, $row->rsm_list_id);
				}
			}
		}
	}
	
	public function onRsformFrontendSubmissionConfirmation($args)
	{
		$SubmissionId = $args['SubmissionId'];
		if ($submission = $this->_getSubmission($SubmissionId))
		{
			$formId = $submission->FormId;
			if ($row = $this->_getRow($formId))
			{
				if ($row->rsm_list_id && $row->rsm_confirm)
				{
					$this->_doAction($row, $SubmissionId);
				}
			}
		}
	}
	
	public function onRsformFrontendAfterFormProcess($args)
	{
		$formId 		= (int) $args['formId'];
		$SubmissionId 	= (int) $args['SubmissionId'];
		
		JFactory::getLanguage()->load('com_rsmail', JPATH_SITE);
		
		if ($row = $this->_getRow($formId))
		{
			// No list selected.
			if (!$row->rsm_list_id)
			{
				return;
			}
			// Confirmation by email enabled.
			if ($row->rsm_confirm)
			{
				return;
			}

			$form = JFactory::getApplication()->input->get('form', array(),'array');
			$this->_doAction($row, $SubmissionId, $form);
		}
	}
	
	/**
	 *	Form Validation
	 */
	public function onRsformFrontendBeforeFormValidation($args)
	{
		$formId = (int) $args['formId'];
		$form	= $args['post'];
		JFactory::getLanguage()->load('com_rsmail',JPATH_SITE);
		
		if ($row = $this->_getRow($formId))
		{
			if (empty($row->rsm_merge_vars['rsm_email']))
			{
				return false;
			}

			if (file_exists(JPATH_SITE.'/components/com_rsmail/helpers/actions.php'))
			{
				require_once JPATH_SITE.'/components/com_rsmail/helpers/actions.php';

				$email_field = $row->rsm_merge_vars['rsm_email'];
				
				if (!isset($email_field))
				{
					$args['invalid'][] = RSFormProHelper::getComponentId($email_field, $formId);
				}
				
				// Get a new instance of the RSMail! helper
				$rsmail = new rsmHelper();
				
				$email_address = isset($form[$row->rsm_merge_vars['rsm_email']]) ? $form[$row->rsm_merge_vars['rsm_email']] : '';
				
				// Subscribe action - Subscribe, Unsubscribe or Let the user choose
				$subscribe = $this->_getAction($row, $form);
				
				if ($subscribe && $row->rsm_action_type == 1)
				{
					if ($rsmail->isSubscribed($email_address, $row->rsm_list_id))
					{
						JFactory::getApplication()->enqueueMessage(JText::_('RSM_ALREADY_SUBSCRIBED'), 'warning');
						$args['invalid'][] = RSFormProHelper::getComponentId($email_field, $formId);
					}
				}
			}
		}
	}
	
	public function onRsformBackendSwitchTasks()
	{
		$app         = JFactory::getApplication();
		$plugin_task = $app->input->getCmd('plugin_task');
		
		if ($plugin_task == 'rsmail.get_merge_vars')
		{
			if (!file_exists(JPATH_SITE.'/components/com_rsmail/helpers/actions.php'))
			{
				return false;
			}

			require_once JPATH_SITE.'/components/com_rsmail/helpers/actions.php';

			if ($list = JFactory::getApplication()->input->getInt('list_id'))
			{
				// Get a new instance of the RSMail! helper
				$rsmail = new rsmHelper();
				// Get list fields
				$results = $rsmail->getFields($list);
			
				echo 'rsm_email_'.JText::_('RSFP_RSM_EMAIL');
				echo is_array($results) && !empty($results) ? "\n".implode("\n",$results) : '';
			}

			$app->close();
		}
	}
	
	protected function _getFields($formId)
	{
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true)
			->select($db->qn('p.PropertyValue'))
			->from($db->qn('#__rsform_components', 'c'))
			->join('LEFT', $db->qn('#__rsform_properties', 'p') . ' ON (' . $db->qn('c.ComponentId') . '=' . $db->qn('p.ComponentId') . ')')
			->where($db->qn('c.FormId') . '=' . $db->q($formId))
			->where($db->qn('p.PropertyName') . '=' . $db->q('NAME'))
			->order($db->qn('c.Order') . ' ' . $db->escape('ASC'));

		return $db->setQuery($query)->loadColumn();
	}
	
	protected function _escapeCommas(&$item)
	{
		$item = str_replace(',', '\,', $item);
	}
	
	public function onRsformFormDelete($formId)
	{
		if ($row = $this->getTable())
		{
			$row->delete($formId);
		}
	}
	
	public function onRsformFormBackup($form, $xml, $fields)
	{
		if ($row = $this->getTable())
		{
			if ($row->load($form->FormId))
			{
				$row->check();

				$data = $row->getProperties();
				unset($data['form_id']);

				$xml->add('rsmail');
				foreach ($data as $property => $value)
				{
					$xml->add($property, $value);
				}
				$xml->add('/rsmail');
			}
		}
	}
	
	public function onRsformFormRestore($form, $xml, $fields)
	{
		if (isset($xml->rsmail))
		{
			$data = array(
				'form_id' => $form->FormId
			);
			
			foreach ($xml->rsmail->children() as $property => $value)
			{
				$data[$property] = (string) $value;
			}
			
			$row = $this->getTable();
			$row->save($data);
		}
	}
	
	public function onRsformBackendFormRestoreTruncate()
	{
		JFactory::getDbo()->truncateTable('#__rsform_rsmail');
	}
}